@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012 Adam Reichold
+Copyright 2012-2013 Adam Reichold
 Copyright 2012 Michał Trybus
 
 This file is part of qpdfview.
@@ -20,7 +20,21 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 */
 
-#include "mainwindow.h"
+#include <iostream>
+
+#include <QApplication>
+#include <QDebug>
+#include <QDir>
+#include <QMessageBox>
+#include <QScopedPointer>
+#include <QTranslator>
+
+#ifdef WITH_DBUS
+
+#include <QDBusInterface>
+#include <QDBusReply>
+
+#endif // WITH_DBUS
 
 #ifdef WITH_SYNCTEX
 
@@ -28,6 +42,15 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 
 #endif // WITH_SYNCTEX
 
+#include "documentview.h"
+#include "mainwindow.h"
+
+#ifdef WITH_SIGNALS
+
+#include "signalhandler.h"
+
+#endif // WITH_SIGNALS
+
 struct File
 {
     QString filePath;
@@ -45,7 +68,7 @@ struct File
 int main(int argc, char** argv)
 {
     qRegisterMetaType< QList< QRectF > >("QList<QRectF>");
-    qRegisterMetaType< Poppler::Page::Rotation >("Poppler::Page::Rotation");
+    qRegisterMetaType< Rotation >("Rotation");
 
     QApplication application(argc, argv);
 
@@ -53,7 +76,7 @@ int main(int argc, char** argv)
     QApplication::setOrganizationName("qpdfview");
     QApplication::setApplicationName("qpdfview");
 
-    QApplication::setApplicationVersion("0.3.7");
+    QApplication::setApplicationVersion(APPLICATION_VERSION);
 
     QApplication::setWindowIcon(QIcon(":icons/qpdfview.svg"));
 
@@ -61,19 +84,25 @@ int main(int argc, char** argv)
 
 #if QT_VERSION >= QT_VERSION_CHECK(4,8,0)
 
-    if(translator.load(QLocale::system(), "qpdfview", "_", DATA_INSTALL_PATH)) { application.installTranslator(&translator); }
-    else if(translator.load(QLocale::system(), "qpdfview", "_", QDir(QApplication::applicationDirPath()).filePath("data"))) { application.installTranslator(&translator); }
+    if(translator.load(QLocale::system(), "qpdfview", "_", QDir(QApplication::applicationDirPath()).filePath("data"))) { application.installTranslator(&translator); }
+    else if(translator.load(QLocale::system(), "qpdfview", "_", DATA_INSTALL_PATH)) { application.installTranslator(&translator); }
 
 #else
 
-    if(translator.load("qpdfview_" + QLocale::system().name(), DATA_INSTALL_PATH)) { application.installTranslator(&translator); }
-    else if(translator.load("qpdfview_" + QLocale::system().name(), QDir(QApplication::applicationDirPath()).filePath("data"))) { application.installTranslator(&translator); }
+    if(translator.load("qpdfview_" + QLocale::system().name(), QDir(QApplication::applicationDirPath()).filePath("data"))) { application.installTranslator(&translator); }
+    else if(translator.load("qpdfview_" + QLocale::system().name(), DATA_INSTALL_PATH)) { application.installTranslator(&translator); }
 
 #endif // QT_VERSION
 
     bool unique = false;
+    bool quiet = false;
+
     bool instanceNameIsNext = false;
     QString instanceName = "";
+
+    bool searchTextIsNext = false;
+    QString searchText = "";
+
     QList< File > files;
 
     {
@@ -89,27 +118,61 @@ int main(int argc, char** argv)
             arguments.removeFirst();
         }
 
-        foreach(QString argument, arguments)
+        foreach(const QString& argument, arguments)
         {
             if(instanceNameIsNext)
             {
                 if(argument.isEmpty())
                 {
-                    qWarning() << QObject::tr("An empty instance name is not allowed.");
+                    qCritical() << QObject::tr("An empty instance name is not allowed.");
                     return 1;
                 }
 
                 instanceNameIsNext = false;
                 instanceName = argument;
             }
+            else if(searchTextIsNext)
+            {
+                if(argument.isEmpty())
+                {
+                    qCritical() << QObject::tr("An empty search text is not allowed.");
+                    return 1;
+                }
+
+                searchTextIsNext = false;
+                searchText = argument;
+            }
             else if(argument == "--unique")
             {
                 unique = true;
             }
+            else if(argument == "--quiet")
+            {
+                quiet = true;
+            }
             else if(argument == "--instance")
             {
                 instanceNameIsNext = true;
             }
+            else if(argument == "--search")
+            {
+                searchTextIsNext = true;
+            }
+            else if(argument == "--help")
+            {
+                std::cout << "Usage: qpdfview [options] [file[#page]] [file[#src:name:line:column]] ..." << std::endl
+                          << std::endl
+                          << "Available options:" << std::endl
+                          << "  --help                      Show this information" << std::endl
+                          << "  --quiet                     Suppress warning messages when opening files" << std::endl
+                          << "  --search text               Search for text in the current tab" << std::endl
+                          << "  --unique                    Open files as tabs in unique window" << std::endl
+                          << "  --unique --instance name    Open files as tabs in named instance" << std::endl
+                          << std::endl
+                          << "Please report bugs at \"https://launchpad.net/qpdfview\"." << std::endl;
+
+                return 0;
+            }
             else
             {
                 File file;
@@ -137,13 +200,19 @@ int main(int argc, char** argv)
 
         if(instanceNameIsNext)
         {
-            qWarning() << QObject::tr("Using '--instance' requires an instance name.");
+            qCritical() << QObject::tr("Using '--instance' requires an instance name.");
             return 1;
         }
 
         if(!unique && !instanceName.isEmpty())
         {
-            qWarning() << QObject::tr("Using '--instance' is not allowed without using '--unique'.");
+            qCritical() << QObject::tr("Using '--instance' is not allowed without using '--unique'.");
+            return 1;
+        }
+
+        if(searchTextIsNext)
+        {
+            qCritical() << QObject::tr("Using '--search' requires a search text.");
             return 1;
         }
     }
@@ -210,26 +279,29 @@ int main(int argc, char** argv)
 
         if(unique)
         {
-            QDBusInterface* interface = new QDBusInterface(serviceName, "/MainWindow", "local.qpdfview.MainWindow", QDBusConnection::sessionBus());
+            QScopedPointer< QDBusInterface > interface(new QDBusInterface(serviceName, "/MainWindow", "local.qpdfview.MainWindow", QDBusConnection::sessionBus()));
 
             if(interface->isValid())
             {
                 interface->call("raiseAndActivate");
 
-                foreach(File file, files)
+                foreach(const File& file, files)
                 {
-                    QDBusReply< bool > reply = interface->call("jumpToPageOrOpenInNewTab", file.filePath, file.page, true, file.enclosingBox);
+                    QDBusReply< bool > reply = interface->call("jumpToPageOrOpenInNewTab", file.filePath, file.page, true, file.enclosingBox, quiet);
 
                     if(!reply.isValid())
                     {
-                        qDebug() << QDBusConnection::sessionBus().lastError().message();
+                        qCritical() << QDBusConnection::sessionBus().lastError().message();
 
-                        delete interface;
                         return 1;
                     }
                 }
 
-                delete interface;
+                if(!searchText.isEmpty())
+                {
+                    interface->call("startSearch", searchText);
+                }
+
                 return 0;
             }
             else
@@ -240,7 +312,7 @@ int main(int argc, char** argv)
 
                 if(!QDBusConnection::sessionBus().registerService(serviceName))
                 {
-                    qDebug() << QDBusConnection::sessionBus().lastError().message();
+                    qCritical() << QDBusConnection::sessionBus().lastError().message();
 
                     delete mainWindow;
                     return 1;
@@ -248,7 +320,7 @@ int main(int argc, char** argv)
 
                 if(!QDBusConnection::sessionBus().registerObject("/MainWindow", mainWindow))
                 {
-                    qDebug() << QDBusConnection::sessionBus().lastError().message();
+                    qCritical() << QDBusConnection::sessionBus().lastError().message();
 
                     delete mainWindow;
                     return 1;
@@ -267,12 +339,33 @@ int main(int argc, char** argv)
 
 #endif // WITH_DBUS
 
+#ifdef WITH_SIGNALS
+
+    if(SignalHandler::prepareSignals())
+    {
+        SignalHandler* signalHandler = new SignalHandler(mainWindow);
+
+        QObject::connect(signalHandler, SIGNAL(sigIntReceived()), mainWindow, SLOT(close()));
+        QObject::connect(signalHandler, SIGNAL(sigTermReceived()), mainWindow, SLOT(close()));
+    }
+    else
+    {
+        qWarning() << QObject::tr("Could not prepare signal handler.");
+    }
+
+#endif // WITH_SIGNALS
+
     mainWindow->show();
     mainWindow->setAttribute(Qt::WA_DeleteOnClose);
 
-    foreach(File file, files)
+    foreach(const File& file, files)
     {
-        mainWindow->openInNewTab(file.filePath, file.page, file.enclosingBox);
+        mainWindow->openInNewTab(file.filePath, file.page, file.enclosingBox, quiet);
+    }
+
+    if(!searchText.isEmpty())
+    {
+        mainWindow->startSearch(searchText);
     }
     
     return application.exec();
