@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012 Adam Reichold
+Copyright 2012-2013 Adam Reichold
 Copyright 2012 Michał Trybus
 Copyright 2012 Alexander Volkov
 
@@ -24,36 +24,36 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef MAINWINDOW_H
 #define MAINWINDOW_H
 
-#include <QtCore>
-#include <QtGui>
-
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-
-#include <QtWidgets>
-#include <QtPrintSupport>
-
-#endif // QT_VERSION
+#include <QMainWindow>
 
 #ifdef WITH_SQL
 
-#include <QtSql>
+#include <QSqlDatabase>
 
 #endif // WITH_SQL
 
 #ifdef WITH_DBUS
 
-#include <QtDBus>
+#include <QDBusAbstractAdaptor>
 
 #endif // WITH_DBUS
 
-#include "documentview.h"
+class QCheckBox;
+class QGraphicsView;
+class QModelIndex;
+class QShortcut;
+class QTableView;
+class QWidgetAction;
 
+#include "global.h"
+
+class Settings;
+class DocumentView;
 class TabWidget;
 class TreeView;
 class ComboBox;
 class SpinBox;
 class ProgressLineEdit;
-class Settings;
 class RecentlyUsedMenu;
 class BookmarkMenu;
 
@@ -68,10 +68,12 @@ public:
     QMenu* createPopupMenu();
 
 public slots:
-    bool open(const QString& filePath, int page = -1, const QRectF& highlight = QRectF());
-    bool openInNewTab(const QString& filePath, int page = -1, const QRectF& highlight = QRectF());
+    bool open(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
+    bool openInNewTab(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
 
-    bool jumpToPageOrOpenInNewTab(const QString& filePath, int page = -1, bool refreshBeforeJump = false, const QRectF& highlight = QRectF());
+    bool jumpToPageOrOpenInNewTab(const QString& filePath, int page = -1, bool refreshBeforeJump = false, const QRectF& highlight = QRectF(), bool quiet = false);
+
+    void startSearch(const QString& text);
 
 protected slots:
     void on_tabWidget_currentChanged(int index);
@@ -81,18 +83,24 @@ protected slots:
     void on_currentTab_numberOfPagesChaned(int numberOfPages);
     void on_currentTab_currentPageChanged(int currentPage);
 
+    void on_currentTab_canJumpChanged(bool backward, bool forward);
+
     void on_currentTab_continuousModeChanged(bool continuousMode);
-    void on_currentTab_layoutModeChanged(DocumentView::LayoutMode layoutMode);
-    void on_currentTab_scaleModeChanged(DocumentView::ScaleMode scaleMode);
+    void on_currentTab_layoutModeChanged(LayoutMode layoutMode);
+    void on_currentTab_scaleModeChanged(ScaleMode scaleMode);
     void on_currentTab_scaleFactorChanged(qreal scaleFactor);
-    void on_currentTab_rotationChanged(Poppler::Page::Rotation rotation);
+    void on_currentTab_rotationChanged(Rotation rotation);
 
+    void on_currentTab_linkClicked(const QString& filePath, int page);
+
+    void on_currentTab_invertColorsChanged(bool invertColors);
     void on_currentTab_highlightAllChanged(bool highlightAll);
-    void on_currentTab_rubberBandModeChanged(PageItem::RubberBandMode rubberBandMode);
+    void on_currentTab_rubberBandModeChanged(RubberBandMode rubberBandMode);
 
-    void on_currentTab_searchProgressed(int progress);
     void on_currentTab_searchFinished();
-    void on_currentTab_searchCanceled();
+    void on_currentTab_searchProgressChanged(int progress);
+
+    void on_currentTab_customContextMenuRequested(const QPoint& pos);
 
     void on_currentPage_editingFinished();
     void on_currentPage_returnPressed();
@@ -117,6 +125,9 @@ protected slots:
 
     void on_jumpToPage_triggered();
 
+    void on_jumpBackward_triggered();
+    void on_jumpForward_triggered();
+
     void on_search_triggered();
     void on_search_returnPressed(const Qt::KeyboardModifiers& modifiers);
     void on_search_timeout();
@@ -124,8 +135,8 @@ protected slots:
     void on_findNext_triggered();
     void on_cancelSearch_triggered();
 
-    void on_copyToClipboard_triggered(bool checked);
-    void on_addAnnotation_triggered(bool checked);
+    void on_copyToClipboardMode_triggered(bool checked);
+    void on_addAnnotationMode_triggered(bool checked);
 
     void on_settings_triggered();
 
@@ -137,12 +148,14 @@ protected slots:
     void on_zoomIn_triggered();
     void on_zoomOut_triggered();
     void on_originalSize_triggered();
-    void on_fitToPageWidth_triggered(bool checked);
-    void on_fitToPageSize_triggered(bool checked);
+    void on_fitToPageWidthMode_triggered(bool checked);
+    void on_fitToPageSizeMode_triggered(bool checked);
 
     void on_rotateLeft_triggered();
     void on_rotateRight_triggered();
 
+    void on_invertColors_triggered(bool checked);
+
     void on_fonts_triggered();
 
     void on_fullscreen_triggered(bool checked);
@@ -154,7 +167,8 @@ protected slots:
     void on_closeAllTabs_triggered();
     void on_closeAllTabsButCurrentTab_triggered();
 
-    void on_tab_triggered();
+    void on_tabAction_triggered();
+    void on_tabShortcut_activated();
 
     void on_previousBookmark_triggered();
     void on_nextBookmark_triggered();
@@ -171,6 +185,8 @@ protected slots:
 
     void on_highlightAll_clicked(bool checked);
 
+    void on_model_reset();
+
     void on_outline_clicked(const QModelIndex& index);
 
     void on_thumbnails_verticalScrollBar_valueChanged(int value);
@@ -182,7 +198,7 @@ protected:
     void dropEvent(QDropEvent* event);
 
 private:
-    Settings* m_settings;
+    static Settings* s_settings;
 
     TabWidget* m_tabWidget;
 
@@ -191,10 +207,15 @@ private:
 
     bool senderIsCurrentTab() const;
 
+    QString windowTitleSuffixForCurrentTab() const;
+
     BookmarkMenu* bookmarkForCurrentTab() const;
 
     SpinBox* m_currentPageSpinBox;
+    QWidgetAction* m_currentPageAction;
+
     ComboBox* m_scaleFactorComboBox;
+    QWidgetAction* m_scaleFactorAction;
 
     ProgressLineEdit* m_searchProgressLineEdit;
     QTimer* m_searchTimer;
@@ -218,13 +239,16 @@ private:
 
     QAction* m_jumpToPageAction;
 
+    QAction* m_jumpBackwardAction;
+    QAction* m_jumpForwardAction;
+
     QAction* m_searchAction;
     QAction* m_findPreviousAction;
     QAction* m_findNextAction;
     QAction* m_cancelSearchAction;
 
-    QAction* m_copyToClipboardAction;
-    QAction* m_addAnnotationAction;
+    QAction* m_copyToClipboardModeAction;
+    QAction* m_addAnnotationModeAction;
 
     QAction* m_settingsAction;
 
@@ -236,12 +260,15 @@ private:
     QAction* m_zoomInAction;
     QAction* m_zoomOutAction;
     QAction* m_originalSizeAction;
-    QAction* m_fitToPageWidthAction;
-    QAction* m_fitToPageSizeAction;
+
+    QAction* m_fitToPageWidthModeAction;
+    QAction* m_fitToPageSizeModeAction;
 
     QAction* m_rotateLeftAction;
     QAction* m_rotateRightAction;
 
+    QAction* m_invertColorsAction;
+
     QAction* m_fontsAction;
 
     QAction* m_fullscreenAction;
@@ -249,12 +276,16 @@ private:
 
     QAction* m_previousTabAction;
     QAction* m_nextTabAction;
+
     QAction* m_closeTabAction;
     QAction* m_closeAllTabsAction;
     QAction* m_closeAllTabsButCurrentTabAction;
 
+    QShortcut* m_tabShortcuts[9];
+
     QAction* m_previousBookmarkAction;
     QAction* m_nextBookmarkAction;
+
     QAction* m_addBookmarkAction;
     QAction* m_removeBookmarkAction;
     QAction* m_removeAllBookmarksAction;
@@ -262,6 +293,9 @@ private:
     QAction* m_contentsAction;
     QAction* m_aboutAction;
 
+    QAction* createAction(const QString& text, const QString& objectName, const QIcon& icon, const QKeySequence& shortcut, const char* member, bool checkable = false);
+    QAction* createAction(const QString& text, const QString& objectName, const QString& iconName, const QKeySequence& shortcut, const char* member, bool checkable = false);
+
     void createActions();
 
     QToolBar* m_fileToolBar;
@@ -270,6 +304,8 @@ private:
 
     QToolBar* m_searchToolBar;
 
+    QToolBar* createToolBar(const QString& text, const QString& objectName, const QStringList& actionNames, const QList< QAction* >& actions);
+
     void createToolBars();
 
     QDockWidget* m_outlineDock;
@@ -281,6 +317,8 @@ private:
     QDockWidget* m_thumbnailsDock;
     QGraphicsView* m_thumbnailsView;
 
+    QDockWidget* createDock(const QString& text, const QString& objectName, const QKeySequence& toggleViewShortcut);
+
     void createDocks();
 
     QMenu* m_fileMenu;
@@ -302,11 +340,9 @@ private:
     void createDatabase();
 
     void restoreTabs();
-    void restoreTabsFromXml(bool removeFile);
     void saveTabs();
 
     void restoreBookmarks();
-    void restoreBookmarksFromXml(bool removeFile);
     void saveBookmarks();
 
     void restorePerFileSettings(DocumentView* tab);
@@ -325,10 +361,12 @@ public:
     explicit MainWindowAdaptor(MainWindow* mainWindow);
 
 public slots:
-    bool open(const QString& filePath, int page = -1, const QRectF& highlight = QRectF());
-    bool openInNewTab(const QString& filePath, int page = -1, const QRectF& highlight = QRectF());
+    bool open(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
+    bool openInNewTab(const QString& filePath, int page = -1, const QRectF& highlight = QRectF(), bool quiet = false);
 
-    bool jumpToPageOrOpenInNewTab(const QString& filePath, int page = -1, bool refreshBeforeJump = false, const QRectF& highlight = QRectF());
+    bool jumpToPageOrOpenInNewTab(const QString& filePath, int page = -1, bool refreshBeforeJump = false, const QRectF& highlight = QRectF(), bool quiet = false);
+
+    Q_NOREPLY void startSearch(const QString& text);
 
     Q_NOREPLY void raiseAndActivate();
 
