@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012 Adam Reichold
+Copyright 2012-2013 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -22,14 +22,29 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef MISCELLANEOUS_H
 #define MISCELLANEOUS_H
 
-#include <QtCore>
-#include <QtGui>
+#include <QComboBox>
+#include <QGraphicsEffect>
+#include <QLineEdit>
+#include <QPainter>
+#include <QSpinBox>
+#include <QTreeView>
 
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
+// graphics composition mode effect
 
-#include <QtWidgets>
+class GraphicsCompositionModeEffect : public QGraphicsEffect
+{
+    Q_OBJECT
 
-#endif // QT_VERSION
+public:
+    GraphicsCompositionModeEffect(QPainter::CompositionMode compositionMode, QObject* parent = 0);
+
+protected:
+    void draw(QPainter* painter);
+
+private:
+    QPainter::CompositionMode m_compositionMode;
+
+};
 
 // tab bar
 
