@@ -1,6 +1,6 @@
 /*
 
-Copyright 2012 Adam Reichold
+Copyright 2012-2013 Adam Reichold
 
 This file is part of qpdfview.
 
@@ -22,14 +22,7 @@ along with qpdfview.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef RECENTLYUSEDMENU_H
 #define RECENTLYUSEDMENU_H
 
-#include <QtCore>
-#include <QtGui>
-
-#if QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-
-#include <QtWidgets>
-
-#endif // QT_VERSION
+#include <QMenu>
 
 class RecentlyUsedMenu : public QMenu
 {
